import 'package:account_kit/constants.dart';
import 'package:account_kit/entities/accessToken/accessToken.dart';
import 'package:built_collection/built_collection.dart';
import 'package:http/http.dart';
import 'package:puma_core/redux/core/coreAction.dart';
import 'package:puma_network/redux/api/apiAction.dart';
import 'package:redux/redux.dart';

// Get AccessTokens
class GetAccessTokensAction extends GetAction {
  static const String _PATH = "/account/accessTokens";
  final String dispatcherKey = constDispatcherKey;
  GetAccessTokensAction() : super(_PATH);

  decodeResponse(Store store, Response data) {
    BuiltList<AccessToken> accessTokens = AccessToken.listFromJson(data.body);
    store.dispatch(GetAccessTokensSuccessAction(accessTokens));
  }
}

class GetAccessTokensSuccessAction extends CoreAction {
  final BuiltList<AccessToken> accessTokens;
  final int timestamp;
  final String dispatcherKey = constDispatcherKey;
  GetAccessTokensSuccessAction(this.accessTokens)
      : this.timestamp = DateTime.now().millisecondsSinceEpoch;
}

// Get AccessToken By ID
class GetAccessTokenByIDAction extends GetAction {
  static const String _PATH = "/account/accessTokens";
  final String dispatcherKey = constDispatcherKey;
  final int accessTokenID;

  GetAccessTokenByIDAction(accessTokenID)
      : accessTokenID = accessTokenID,
        super("$_PATH/$accessTokenID");

  decodeResponse(Store store, Response data) {
    AccessToken accessToken = AccessToken.fromJson(data.body)!;
    store.dispatch(GetAccessTokenByIDSuccessAction(accessToken));
  }
}

class GetAccessTokenByIDSuccessAction extends CoreAction {
  final AccessToken accessToken;
  final int timestamp;
  final String dispatcherKey = constDispatcherKey;
  GetAccessTokenByIDSuccessAction(this.accessToken)
      : this.timestamp = DateTime.now().millisecondsSinceEpoch;
}

// Update AccessToken
class UpdateAccessTokenAction extends PutAction {
  static const String _PATH = "/account/accessTokens";
  final int accessTokenID;
  final Map<dynamic, dynamic> body;
  final String dispatcherKey = constDispatcherKey;

  UpdateAccessTokenAction(this.accessTokenID, this.body)
      : super(
          "$_PATH/$accessTokenID",
          body,
        );

  decodeResponse(Store store, Response data) {
    store.dispatch(UpdateAccessTokenSuccessAction(accessTokenID, body));
  }
}

class UpdateAccessTokenSuccessAction extends CoreAction {
  final int accessTokenID;
  final Map<dynamic, dynamic> body;
  final int timestamp;
  final String dispatcherKey = constDispatcherKey;
  UpdateAccessTokenSuccessAction(this.accessTokenID, this.body)
      : this.timestamp = DateTime.now().millisecondsSinceEpoch;
}

// Create AccessToken
class CreateAccessTokenAction extends PostAction {
  static const String _PATH = "/account/users/current/accessTokens";
  final Map<dynamic, dynamic> body;
  final String dispatcherKey = constDispatcherKey;

  CreateAccessTokenAction(body)
      : body = body,
        super("$_PATH", body);

  decodeResponse(Store store, Response data) {
    AccessToken accessToken = AccessToken.fromJson(data.body)!;
    store.dispatch(CreateAccessTokenSuccessAction(accessToken));
  }
}

class CreateAccessTokenSuccessAction extends CoreAction {
  final AccessToken accessToken;
  final int timestamp;
  final String dispatcherKey = constDispatcherKey;
  CreateAccessTokenSuccessAction(this.accessToken)
      : this.timestamp = DateTime.now().millisecondsSinceEpoch;
}

// Delete AccessToken
class DeleteAccessTokenAction extends DeleteAction {
  static const String _PATH = "/account/accessTokens";
  final int accessTokenID;
  final String dispatcherKey = constDispatcherKey;

  DeleteAccessTokenAction(this.accessTokenID) : super("$_PATH/$accessTokenID");

  decodeResponse(Store store, Response data) {
    store.dispatch(DeleteAccessTokenSuccessAction(accessTokenID));
  }
}

class DeleteAccessTokenSuccessAction extends CoreAction {
  final int accessTokenID;
  final int timestamp;
  final String dispatcherKey = constDispatcherKey;
  DeleteAccessTokenSuccessAction(this.accessTokenID)
      : this.timestamp = DateTime.now().millisecondsSinceEpoch;
}

class SetBiometricAccessToken extends CoreAction {
  final AccessToken biometricAccessToken;
  final int timestamp;
  final String dispatcherKey = constDispatcherKey;
  SetBiometricAccessToken(this.biometricAccessToken)
      : this.timestamp = DateTime.now().millisecondsSinceEpoch;
}

class RemoveBiometricAccessToken extends CoreAction {
  final int timestamp;
  final String dispatcherKey = constDispatcherKey;
  RemoveBiometricAccessToken()
      : this.timestamp = DateTime.now().millisecondsSinceEpoch;
}
