import 'package:account_kit/redux/accessToken/accessTokenActions.dart';
import 'package:redux/redux.dart';

import 'accessTokenState.dart';

final accessTokenReducer = combineReducers<AccessTokenState>([
  TypedReducer<AccessTokenState, GetAccessTokensSuccessAction>(
      _getAccessTokensSuccess),
  TypedReducer<AccessTokenState, GetAccessTokenByIDSuccessAction>(
      _getAccessTokenByIDSuccess),
  TypedReducer<AccessTokenState, DeleteAccessTokenSuccessAction>(
      _deleteAccessTokenSuccess),
  TypedReducer<AccessTokenState, CreateAccessTokenSuccessAction>(
      _createAccessTokenSuccess),
  TypedReducer<AccessTokenState, UpdateAccessTokenSuccessAction>(
      _updateAccessTokenSuccess),
  TypedReducer<AccessTokenState, SetBiometricAccessToken>(
      _setBiometricAccessToken),
  TypedReducer<AccessTokenState, RemoveBiometricAccessToken>(
      _removeBiometricAccessToken),
]);

// Remove Biometric AccessTokens
AccessTokenState _removeBiometricAccessToken(
    AccessTokenState state, RemoveBiometricAccessToken action) {
  return state.rebuild(
    (b) => b..biometricAccessToken = null,
  );
}

// Remove Biometric AccessTokens
AccessTokenState _setBiometricAccessToken(
    AccessTokenState state, SetBiometricAccessToken action) {
  return state.rebuild(
    (b) => b..biometricAccessToken = action.biometricAccessToken.toBuilder(),
  );
}

// Get AccessTokens
AccessTokenState _getAccessTokensSuccess(
    AccessTokenState state, GetAccessTokensSuccessAction action) {
  return state.rebuild(
    (b) => b..accessTokens = action.accessTokens.toBuilder(),
  );
}

// Get AccessToken By ID
AccessTokenState _getAccessTokenByIDSuccess(
    AccessTokenState state, GetAccessTokenByIDSuccessAction action) {
  return state.rebuild(
    (b) => b..accessTokensByID[action.accessToken.id] = action.accessToken,
  );
}

// Update AccessToken
AccessTokenState _updateAccessTokenSuccess(
    AccessTokenState state, UpdateAccessTokenSuccessAction action) {
  return state;
}

// Create AccessToken
AccessTokenState _createAccessTokenSuccess(
    AccessTokenState state, CreateAccessTokenSuccessAction action) {
  return state.rebuild(
    (b) => b..accessTokensByID[action.accessToken.id] = action.accessToken,
  );
}

// Delete AccessToken
AccessTokenState _deleteAccessTokenSuccess(
    AccessTokenState state, DeleteAccessTokenSuccessAction action) {
  return state;
}
