import 'package:account_kit/constants.dart';
import 'package:account_kit/redux/accessToken/accessTokenActions.dart';
import 'package:account_kit/redux/accountKitState.dart';
import 'package:account_kit/redux/user/userActions.dart';
import 'package:puma_network/redux/api/apiAction.dart';
import 'package:puma_network/redux/network/networkAction.dart';
import 'package:redux/redux.dart';

class AccountKitMiddleware<T extends VirtualAccountState> {
  Middleware<T> createMiddleware() {
    return (Store<T> store, dynamic action, NextDispatcher next) {
      print(store.state.accountKitState.prefix);

      if (action is StopImpersonateAction) {
        store.dispatch(
            SetImpersonateTokenNetworkAction(authImpersonateToken: null));
      }

      /// Token is cleared after a signout
      if ((action is SignOutAction)) {
        store.dispatch(ClearTokenNetworkAction());
      }

      /// Provides path to the request ONLY for accountkit actions
      if (action is RequestAction) {
        RequestAction castAction = action;
        if (castAction.dispatcherKey == constDispatcherKey) {
          castAction.path =
              "${store.state.accountKitState.prefix}${castAction.path}";
        }
        return next(castAction);
      }

      if (action is ImpersonateSuccessAction) {
        store.dispatch(SetImpersonateTokenNetworkAction(
            authImpersonateToken: action.authImpersonateToken));
      }

      /// Save the access token as a biometric way to sign in
      if ((action is CreateAccessTokenSuccessAction)) {
        store.dispatch(SetBiometricAccessToken(action.accessToken));
        store
            .dispatch(SetBiometricTokenNetworkAction(action.accessToken.token));
      }
      next(action);
    };
  }
}
