import 'dart:convert';

import 'package:account_kit/redux/accessToken/accessTokenState.dart';
import 'package:account_kit/redux/app/appState.dart';
import 'package:account_kit/redux/group/groupState.dart';
import 'package:account_kit/redux/organization/organizationState.dart';
import 'package:account_kit/redux/user/userState.dart';
import 'package:account_kit/serializers.dart';
import 'package:built_value/built_value.dart';
import 'package:built_value/serializer.dart';
import 'package:puma_network/redux/network/networkState.dart';
import 'package:puma_ui/redux/uiState.dart';

part 'accountKitState.g.dart';

abstract class VirtualAccountState
    implements VirtualUiState, VirtualNetworkState {
  AccountKitState get accountKitState;
  NetworkState get networkState;
}

abstract class AccountKitState
    implements Built<AccountKitState, AccountKitStateBuilder> {
  AccountKitState._();
  factory AccountKitState([updates(AccountKitStateBuilder b)]) =
      _$AccountKitState;

  static AccountKitState initial() {
    return new AccountKitState((b) => b
      ..appState.replace(AppState.initial())
      ..userState.replace(UserState.initial())
      ..groupState.replace(GroupState.initial())
      ..accessTokenState.replace(AccessTokenState.initial())
      ..organizationState.replace(OrganizationState.initial())
      ..prefix = "");
  }

  AppState get appState;
  UserState get userState;
  GroupState get groupState;
  OrganizationState get organizationState;
  AccessTokenState get accessTokenState;
  String get prefix;

  set prefix(String prefix) => this.prefix = prefix;

  String toJson() {
    return json
        .encode(serializers.serializeWith(AccountKitState.serializer, this));
  }

  static AccountKitState? fromJson(jsonString) {
    return serializers.deserializeWith(
        AccountKitState.serializer, json.decode(jsonString));
  }

  static Serializer<AccountKitState> get serializer =>
      _$accountKitStateSerializer;
}
