import 'package:account_kit/constants.dart';
import 'package:account_kit/entities/app/app.dart';
import 'package:built_collection/built_collection.dart';
import 'package:http/http.dart';
import 'package:puma_network/redux/api/apiAction.dart';
import 'package:redux/redux.dart';

// Get Apps
class GetAppsAction extends GetAction {
  static const String _PATH = "/account/apps";
  final String dispatcherKey = constDispatcherKey;
  GetAppsAction() : super(_PATH);

  decodeResponse(Store store, Response data) {
    BuiltList<App> apps = App.listFromJson(data.body);
    store.dispatch(GetAppsSuccessAction(apps));
  }
}

class GetAppsSuccessAction {
  final String dispatcherKey = constDispatcherKey;
  final BuiltList<App> apps;
  GetAppsSuccessAction(this.apps);
}

// Get App By ID
class GetAppByIDAction extends GetAction {
  static const String _PATH = "/account/apps";
  final String dispatcherKey = constDispatcherKey;
  final int appID;

  GetAppByIDAction(appID)
      : appID = appID,
        super("$_PATH/$appID");

  decodeResponse(Store store, Response data) {
    App app = App.fromJson(data.body)!;
    store.dispatch(GetAppByIDSuccessAction(app));
  }
}

class GetAppByIDSuccessAction {
  final String dispatcherKey = constDispatcherKey;
  final App app;

  GetAppByIDSuccessAction(this.app);
}

// Update App
class UpdateAppAction extends PutAction {
  static const String _PATH = "/account/apps";
  final String dispatcherKey = constDispatcherKey;
  final int appID;
  final Map<dynamic, dynamic> body;

  UpdateAppAction(this.appID, this.body)
      : super(
          "$_PATH/$appID",
          body,
        );

  decodeResponse(Store store, Response data) {
    store.dispatch(UpdateAppSuccessAction(appID, body));
  }
}

class UpdateAppSuccessAction {
  final String dispatcherKey = constDispatcherKey;
  final int appID;
  final Map<dynamic, dynamic> body;

  UpdateAppSuccessAction(this.appID, this.body);
}

// Create App
class CreateAppAction extends PostAction {
  static const String _PATH = "/account/apps";
  final String dispatcherKey = constDispatcherKey;
  final Map<dynamic, dynamic> body;

  CreateAppAction(body)
      : body = body,
        super("$_PATH", body);

  decodeResponse(Store store, Response data) {
    App app = App.fromJson(data.body)!;
    store.dispatch(CreateAppSuccessAction(app));
  }
}

class CreateAppSuccessAction {
  final String dispatcherKey = constDispatcherKey;
  final App app;

  CreateAppSuccessAction(this.app);
}

// Delete App
class DeleteAppAction extends DeleteAction {
  static const String _PATH = "/account/apps";
  final String dispatcherKey = constDispatcherKey;
  final int appID;

  DeleteAppAction(this.appID) : super("$_PATH/$appID");

  decodeResponse(Store store, Response data) {
    store.dispatch(DeleteAppSuccessAction(appID));
  }
}

class DeleteAppSuccessAction {
  final String dispatcherKey = constDispatcherKey;
  final int appID;

  DeleteAppSuccessAction(this.appID);
}
