import 'package:account_kit/constants.dart';
import 'package:account_kit/entities/group/group.dart';
import 'package:account_kit/entities/user/user.dart';
import 'package:built_collection/built_collection.dart';
import 'package:http/http.dart';
import 'package:puma_core/redux/core/coreAction.dart';
import 'package:puma_network/redux/api/apiAction.dart';
import 'package:redux/redux.dart';

// Get Groups
class GetGroupsAction extends GetAction {
  static const String _PATH = "/account/groups";
  final String dispatcherKey = constDispatcherKey;
  GetGroupsAction() : super(_PATH);

  decodeResponse(Store store, Response data) {
    BuiltList<Group> groups = Group.listFromJson(data.body);
    store.dispatch(GetGroupsSuccessAction(groups));
  }
}

class GetGroupsSuccessAction extends CoreAction {
  final BuiltList<Group> groups;
  final int timestamp;
  final String dispatcherKey = constDispatcherKey;
  GetGroupsSuccessAction(this.groups)
      : this.timestamp = DateTime.now().millisecondsSinceEpoch;
}

// Get Groups
class GetGroupUsersAction extends GetAction {
  static const String _PATH = "/account/groups";
  final String dispatcherKey = constDispatcherKey;
  final int groupID;

  GetGroupUsersAction({required this.groupID}) : super("$_PATH/$groupID/users");

  decodeResponse(Store store, Response data) {
    BuiltList<User> users = User.listFromJson(data.body);
    store.dispatch(GetGroupUsersSuccessAction(users, groupID));
  }
}

class GetGroupUsersSuccessAction extends CoreAction {
  final BuiltList<User> users;
  final int groupID;
  final int timestamp;
  final String dispatcherKey = constDispatcherKey;
  GetGroupUsersSuccessAction(this.users, this.groupID)
      : this.timestamp = DateTime.now().millisecondsSinceEpoch;
}

// Get Group By ID
class GetGroupByIDAction extends GetAction {
  static const String _PATH = "/account/groups";
  final String dispatcherKey = constDispatcherKey;
  final int groupID;

  GetGroupByIDAction(groupID)
      : groupID = groupID,
        super("$_PATH/$groupID");

  decodeResponse(Store store, Response data) {
    Group group = Group.fromJson(data.body)!;
    store.dispatch(GetGroupByIDSuccessAction(group));
  }
}

class GetGroupByIDSuccessAction extends CoreAction {
  final Group group;
  final int timestamp;
  final String dispatcherKey = constDispatcherKey;
  GetGroupByIDSuccessAction(this.group)
      : this.timestamp = DateTime.now().millisecondsSinceEpoch;
}

// Update Group
class UpdateGroupAction extends PutAction {
  static const String _PATH = "/account/groups";
  final String dispatcherKey = constDispatcherKey;
  final int groupID;
  final Map<dynamic, dynamic> body;

  UpdateGroupAction(this.groupID, this.body)
      : super(
          "$_PATH/$groupID",
          body,
        );

  decodeResponse(Store store, Response data) {
    store.dispatch(UpdateGroupSuccessAction(groupID, body));
  }
}

class UpdateGroupSuccessAction extends CoreAction {
  final int groupID;
  final Map<dynamic, dynamic> body;
  final int timestamp;
  final String dispatcherKey = constDispatcherKey;
  UpdateGroupSuccessAction(this.groupID, this.body)
      : this.timestamp = DateTime.now().millisecondsSinceEpoch;
}

// Create Group
class CreateGroupAction extends PostAction {
  static const String _PATH = "/account/groups";
  final Map<dynamic, dynamic> body;
  final String dispatcherKey = constDispatcherKey;

  CreateGroupAction(body)
      : body = body,
        super("$_PATH", body);

  decodeResponse(Store store, Response data) {
    Group group = Group.fromJson(data.body)!;
    store.dispatch(CreateGroupSuccessAction(group));
  }
}

class CreateGroupSuccessAction extends CoreAction {
  final Group group;
  final int timestamp;
  final String dispatcherKey = constDispatcherKey;
  CreateGroupSuccessAction(this.group)
      : this.timestamp = DateTime.now().millisecondsSinceEpoch;
}

// Delete Group
class DeleteGroupAction extends DeleteAction {
  static const String _PATH = "/account/groups";
  final String dispatcherKey = constDispatcherKey;
  final int groupID;

  DeleteGroupAction(this.groupID) : super("$_PATH/$groupID");

  decodeResponse(Store store, Response data) {
    store.dispatch(DeleteGroupSuccessAction(groupID));
  }
}

class DeleteGroupSuccessAction extends CoreAction {
  final int groupID;
  final int timestamp;
  final String dispatcherKey = constDispatcherKey;
  DeleteGroupSuccessAction(this.groupID)
      : this.timestamp = DateTime.now().millisecondsSinceEpoch;
}
