import 'package:account_kit/entities/user/user.dart';
import 'package:account_kit/redux/group/groupActions.dart';
import 'package:built_collection/built_collection.dart';
import 'package:redux/redux.dart';

import 'groupState.dart';

final groupReducer = combineReducers<GroupState>([
  TypedReducer<GroupState, GetGroupsSuccessAction>(_getGroupsSuccess),
  TypedReducer<GroupState, GetGroupByIDSuccessAction>(_getGroupByIDSuccess),
  TypedReducer<GroupState, DeleteGroupSuccessAction>(_deleteGroupSuccess),
  TypedReducer<GroupState, CreateGroupSuccessAction>(_createGroupSuccess),
  TypedReducer<GroupState, UpdateGroupSuccessAction>(_updateGroupSuccess),
  TypedReducer<GroupState, GetGroupUsersSuccessAction>(
      _getGroupUsersSuccessAction),
]);

// Get Groups
GroupState _getGroupsSuccess(GroupState state, GetGroupsSuccessAction action) {
  return state.rebuild(
    (b) => b..groups = action.groups.toBuilder(),
  );
}

// Get Group By ID
GroupState _getGroupByIDSuccess(
    GroupState state, GetGroupByIDSuccessAction action) {
  return state.rebuild(
    (b) => b..groupsByID[action.group.id!] = action.group,
  );
}

// Update Group
GroupState _updateGroupSuccess(
    GroupState state, UpdateGroupSuccessAction action) {
  return state;
}

// Create Group
GroupState _createGroupSuccess(
    GroupState state, CreateGroupSuccessAction action) {
  return state.rebuild(
    (b) => b..groupsByID[action.group.id!] = action.group,
  );
}

// Delete Group
GroupState _deleteGroupSuccess(
    GroupState state, DeleteGroupSuccessAction action) {
  return state;
}

// Group Users
GroupState _getGroupUsersSuccessAction(
    GroupState state, GetGroupUsersSuccessAction action) {
  if (state.groupUsers != null) {
    return state.rebuild(
      (b) => b..groupUsers[action.groupID] = action.users,
    );
  }

  var gu = MapBuilder<int, BuiltList<User>>();
  gu[action.groupID] = action.users;
  return state.rebuild((b) => b..groupUsers = gu);
}
