import 'dart:convert';

import 'package:account_kit/entities/group/group.dart';
import 'package:account_kit/entities/user/user.dart';
import 'package:account_kit/serializers.dart';
import 'package:built_collection/built_collection.dart';
import 'package:built_value/built_value.dart';
import 'package:built_value/serializer.dart';

part 'groupState.g.dart';

abstract class GroupState implements Built<GroupState, GroupStateBuilder> {
  GroupState._();
  factory GroupState([updates(GroupStateBuilder b)]) = _$GroupState;

  static GroupState initial() {
    return new GroupState((b) => b
      ..groups = null
      ..groupUsers = BuiltMap<int, BuiltList<User>>().toBuilder()
      ..groupsByID = MapBuilder<int, Group>());
  }

  BuiltList<Group>? get groups;
  BuiltMap<int, Group>? get groupsByID;
  BuiltMap<int, BuiltList<User>>? get groupUsers;

  String toJson() {
    return json.encode(serializers.serializeWith(GroupState.serializer, this));
  }

  static GroupState? fromJson(jsonString) {
    return serializers.deserializeWith(
        GroupState.serializer, json.decode(jsonString));
  }

  static Serializer<GroupState> get serializer => _$groupStateSerializer;
}
