import 'package:account_kit/constants.dart';
import 'package:account_kit/entities/organization/organization.dart';
import 'package:built_collection/built_collection.dart';
import 'package:http/http.dart';
import 'package:puma_core/redux/core/coreAction.dart';
import 'package:puma_network/redux/api/apiAction.dart';
import 'package:redux/redux.dart';

// Get Organizations
class GetOrganizationsAction extends GetAction {
  static const String _PATH = "/account/organizations";
  final String dispatcherKey = constDispatcherKey;
  GetOrganizationsAction() : super(_PATH);

  decodeResponse(Store store, Response data) {
    BuiltList<Organization> organizations =
        Organization.listFromJson(data.body);
    store.dispatch(GetOrganizationsSuccessAction(organizations));
  }
}

class GetOrganizationsSuccessAction extends CoreAction {
  final BuiltList<Organization> organizations;
  final int timestamp;
  final String dispatcherKey = constDispatcherKey;
  GetOrganizationsSuccessAction(this.organizations)
      : this.timestamp = DateTime.now().millisecondsSinceEpoch;
}

// Get Organization By ID
class GetOrganizationByIDAction extends GetAction {
  static const String _PATH = "/account/organizations";
  final int organizationID;
  final String dispatcherKey = constDispatcherKey;

  GetOrganizationByIDAction(organizationID)
      : organizationID = organizationID,
        super("$_PATH/$organizationID");

  decodeResponse(Store store, Response data) {
    Organization organization = Organization.fromJson(data.body)!;
    store.dispatch(GetOrganizationByIDSuccessAction(organization));
  }
}

class GetOrganizationByIDSuccessAction extends CoreAction {
  final Organization organization;
  final int timestamp;
  final String dispatcherKey = constDispatcherKey;
  GetOrganizationByIDSuccessAction(this.organization)
      : this.timestamp = DateTime.now().millisecondsSinceEpoch;
}

// Update Organization
class UpdateOrganizationAction extends PutAction {
  static const String _PATH = "/account/organizations";
  final String dispatcherKey = constDispatcherKey;
  final int organizationID;
  final Map<dynamic, dynamic> body;

  UpdateOrganizationAction(this.organizationID, this.body)
      : super(
          "$_PATH/$organizationID",
          body,
        );

  decodeResponse(Store store, Response data) {
    store.dispatch(UpdateOrganizationSuccessAction(organizationID, body));
  }
}

class UpdateOrganizationSuccessAction extends CoreAction {
  final int organizationID;
  final Map<dynamic, dynamic> body;
  final int timestamp;
  final String dispatcherKey = constDispatcherKey;
  UpdateOrganizationSuccessAction(this.organizationID, this.body)
      : this.timestamp = DateTime.now().millisecondsSinceEpoch;
}

// Create Organization
class CreateOrganizationAction extends PostAction {
  static const String _PATH = "/account/organizations";
  final Map<dynamic, dynamic> body;
  final String dispatcherKey = constDispatcherKey;

  CreateOrganizationAction(body)
      : body = body,
        super("$_PATH", body);

  decodeResponse(Store store, Response data) {
    Organization organization = Organization.fromJson(data.body)!;
    store.dispatch(CreateOrganizationSuccessAction(organization));
  }
}

class CreateOrganizationSuccessAction extends CoreAction {
  final Organization organization;
  final int timestamp;
  final String dispatcherKey = constDispatcherKey;
  CreateOrganizationSuccessAction(this.organization)
      : this.timestamp = DateTime.now().millisecondsSinceEpoch;
}

// Delete Organization
class DeleteOrganizationAction extends DeleteAction {
  static const String _PATH = "/account/organizations";
  final int organizationID;
  final String dispatcherKey = constDispatcherKey;

  DeleteOrganizationAction(this.organizationID)
      : super("$_PATH/$organizationID");

  decodeResponse(Store store, Response data) {
    store.dispatch(DeleteOrganizationSuccessAction(organizationID));
  }
}

class DeleteOrganizationSuccessAction extends CoreAction {
  final int organizationID;
  final int timestamp;
  final String dispatcherKey = constDispatcherKey;
  DeleteOrganizationSuccessAction(this.organizationID)
      : this.timestamp = DateTime.now().millisecondsSinceEpoch;
}
