import 'dart:convert';

import 'package:account_kit/entities/organization/organization.dart';
import 'package:account_kit/serializers.dart';
import 'package:built_collection/built_collection.dart';
import 'package:built_value/built_value.dart';
import 'package:built_value/serializer.dart';

part 'organizationState.g.dart';

abstract class OrganizationState
    implements Built<OrganizationState, OrganizationStateBuilder> {
  OrganizationState._();
  factory OrganizationState([updates(OrganizationStateBuilder b)]) =
      _$OrganizationState;

  static OrganizationState initial() {
    return new OrganizationState(
        (b) => b..organizationsByID = MapBuilder<int, Organization>());
  }

  BuiltList<Organization>? get organizations;
  BuiltMap<int, Organization>? get organizationsByID;

  String toJson() {
    return json
        .encode(serializers.serializeWith(OrganizationState.serializer, this));
  }

  static OrganizationState? fromJson(jsonString) {
    return serializers.deserializeWith(
        OrganizationState.serializer, json.decode(jsonString));
  }

  static Serializer<OrganizationState> get serializer =>
      _$organizationStateSerializer;
}
