import 'package:redux/redux.dart';

import 'userActions.dart';
import 'userState.dart';

final userReducer = combineReducers<UserState>([
  TypedReducer<UserState, GetUsersSuccessAction>(_getUsersSuccess),
  TypedReducer<UserState, AccountLoadedAction>(_setUser),
  TypedReducer<UserState, UserRequestLoadedAction>(_setUserRequest),
  TypedReducer<UserState, PostSignin>(_loadingUser),
  TypedReducer<UserState, SignOutAction>(_signOut),
  TypedReducer<UserState, GetUserOpaReadySuccessAction>(_opaReady),
  TypedReducer<UserState, SetCheckingOpa>(_setCheckingOpa),
  TypedReducer<UserState, SetOpaReady>(_setOpaReady),
  TypedReducer<UserState, ClearUserRequestAction>(_clearUserRequest),
]);

UserState _getUsersSuccess(UserState state, GetUsersSuccessAction action) {
  return state.rebuild(
    (b) => b..users = action.users.toBuilder(),
  );
}

UserState _setUser(UserState state, AccountLoadedAction action) {
  return state.rebuild((b) => b
    ..user.replace(action.user)
    ..token = action.user.token ?? state.token);
}

UserState _opaReady(UserState state, GetUserOpaReadySuccessAction action) {
  return state.rebuild((b) => b..opaReady = true);
}

UserState _setUserRequest(UserState state, UserRequestLoadedAction action) {
  return state.rebuild((b) => b..request.replace(action.request));
}

UserState _loadingUser(UserState state, PostSignin action) {
  return state;
}

UserState _signOut(UserState state, SignOutAction action) {
  return UserState.initial();
}

UserState _clearUserRequest(UserState state, ClearUserRequestAction action) {
  return state.rebuild((b) => b..request = null);
}

UserState _setCheckingOpa(UserState state, SetCheckingOpa action) {
  return state.rebuild((b) => b..checkingOpa = action.value);
}

UserState _setOpaReady(UserState state, SetOpaReady action) {
  return state.rebuild((b) => b..opaReady = action.value);
}
