import 'dart:convert';

import 'package:account_kit/entities/request/request.dart';
import 'package:built_collection/built_collection.dart';
import 'package:built_value/built_value.dart';
import 'package:built_value/serializer.dart';

import '../../entities/user/user.dart';
import '../../serializers.dart';

part 'userState.g.dart';

abstract class UserState implements Built<UserState, UserStateBuilder> {
  UserState._();
  factory UserState([updates(UserStateBuilder b)]) = _$UserState;

  static UserState initial() {
    return new UserState((b) => b
      ..token = ""
      ..opaReady = false
      ..users = ListBuilder<User>()
      ..checkingOpa = false);
  }

  User? get user;
  BuiltList<User> get users;
  String? get token;
  Request? get request;
  bool get opaReady;
  bool? get checkingOpa;

  String toJson() {
    return json.encode(serializers.serializeWith(UserState.serializer, this));
  }

  static UserState? fromJson(jsonString) {
    return serializers.deserializeWith(
        UserState.serializer, json.decode(jsonString));
  }

  static Serializer<UserState> get serializer => _$userStateSerializer;
}
