import 'package:account_kit/entities/group/group.dart';
import 'package:account_kit/redux/accountKitState.dart';
import 'package:account_kit/redux/group/groupActions.dart';
import 'package:account_kit/ui/containers/ImpersonateContainer.dart';
import 'package:built_collection/built_collection.dart';
import 'package:flutter/material.dart';
import 'package:flutter_redux/flutter_redux.dart';
import 'package:redux/redux.dart';

class ImpersonateGroupContainer<T extends VirtualAccountState>
    extends StatefulWidget {
  ImpersonateGroupContainer();
  @override
  _ImpersonatingState createState() => _ImpersonatingState<T>();
}

class _ImpersonatingState<T extends VirtualAccountState>
    extends State<ImpersonateGroupContainer> {
  bool searchEnabled = false;
  TextEditingController editingController = TextEditingController();
  var list = List<Group>.empty(growable: true);

  @override
  Widget build(BuildContext context) {
    return StoreConnector<T, _ViewModel>(
      onInitialBuild: (vm) {
        vm.getGroups();
      },
      converter: (Store<T> store) => _ViewModel.create(store),
      builder: (BuildContext context, _ViewModel viewModel) =>
          _createBody(context, viewModel),
    );
  }

  void filterSearchResults(String query, _ViewModel vm) {
    if (query.isEmpty) {
      list = vm.groups?.toList() ?? [];
    } else {
      var _list = List<Group>.empty(growable: true);
      (vm.groups?.toList() ?? []).forEach((group) {
        if (group.name!.toLowerCase().contains(query)) {
          _list.add(group);
        }
      });
      setState(() {
        list = _list;
      });
    }
  }

  Widget _createBody(BuildContext context, _ViewModel vm) {
    final groups = searchEnabled ? list : (vm.groups?.toList() ?? []);
    final isDark = Theme.of(context).brightness == Brightness.dark;

    return Scaffold(
        appBar: AppBar(
          iconTheme: IconThemeData(
            color:
                isDark ? Colors.white : Colors.black, //change your color here
          ),
          backgroundColor: isDark ? Colors.grey.shade900 : Colors.white,
          title: Text(
            "Select Group",
            style: TextStyle(
              color: isDark ? Colors.white : Colors.black,
            ),
          ),
          actions: [
            IconButton(
                icon: Icon(
                  !searchEnabled ? Icons.search : Icons.search_off,
                  color: isDark ? Colors.white : Colors.black,
                ),
                onPressed: () {
                  setState(() {
                    searchEnabled = !searchEnabled;
                  });
                })
          ],
        ),
        body: Column(
          children: [
            if (searchEnabled)
              Padding(
                padding: const EdgeInsets.all(8.0),
                child: TextField(
                  autofocus: true,
                  onChanged: (e) => filterSearchResults(e.toLowerCase(), vm),
                  controller: editingController,
                  decoration: InputDecoration(
                      labelText: "Search Group",
                      hintText: "Search Group",
                      prefixIcon: Icon(Icons.search),
                      border: OutlineInputBorder(
                          borderRadius:
                              BorderRadius.all(Radius.circular(25.0)))),
                ),
              ),
            Expanded(
              child: ListView(
                padding: const EdgeInsets.all(16.0),
                children: List.generate(groups.length, (index) {
                  final group = groups[index];
                  return ListTile(
                    onTap: () => Navigator.push(
                      context,
                      MaterialPageRoute(
                          builder: (context) =>
                              ImpersonateContainer<T>(groupID: group.id)),
                    ),
                    title: Text("${group.name}"),
                    trailing: Icon(Icons.arrow_forward_ios, size: 16),
                  );
                }),
              ),
            ),
          ],
        ));
  }
}

class _ViewModel {
  BuiltList<Group>? groups;
  Function getGroups;
  _ViewModel(
    this.groups,
    this.getGroups,
  );

  factory _ViewModel.create(Store<VirtualAccountState> store) {
    return _ViewModel(
      store.state.accountKitState.groupState.groups,
      () => store.dispatch(GetGroupsAction()),
    );
  }
}
