import 'dart:async';

import 'package:account_kit/redux/accountKitState.dart';
import 'package:account_kit/redux/user/userActions.dart';
import 'package:flutter/material.dart';
import 'package:flutter_redux/flutter_redux.dart';
import 'package:puma_core/extensions/hexColor.dart';
import 'package:redux/redux.dart';

class ImpersonatingBuilder<T extends VirtualAccountState>
    extends StatefulWidget {
  final BuildContext context;
  final Widget? widget;
  ImpersonatingBuilder({
    required this.context,
    this.widget,
  });

  @override
  _ImpersonatingBuilderState createState() => _ImpersonatingBuilderState<T>();
}

class _ImpersonatingBuilderState<T extends VirtualAccountState>
    extends State<ImpersonatingBuilder> {
  final _myDuration = Duration(seconds: 2);
  var red1 = HexColor.fromHex("#ff1744");
  final red2 = HexColor.fromHex("#f44336");
  var _color = Colors.red.shade500;
  var tick = 0;
  Timer? _timer;

  @override
  void dispose() {
    _timer?.cancel();
    super.dispose();
  }

  void startTimer() {
    print("startTimer");
    const duration = const Duration(seconds: 2);
    _timer = new Timer.periodic(
      duration,
      (Timer timer) {
        print("tick");
        setState(() {
          tick = tick + 1;
          _color = tick % 2 == 0 ? red1 : red2;
        });
      },
    );
  }

  final String title = "ImpersonatingBuilder";
  @override
  Widget build(BuildContext context) {
    return StoreConnector<T, _ViewModel>(
      onDidChange: (vm1, vm2) {
        print("onDidChange: ${vm2.authImpersonateToken}, $_timer");
        if (vm2.authImpersonateToken != null && _timer == null) {
          startTimer();
        }

        if (vm2.authImpersonateToken == null && _timer != null) {
          _timer?.cancel();
          setState(() {
            _timer = null;
            tick = 0;
          });
        }
      },
      converter: (Store<T> store) => _ViewModel.create(store),
      builder: (BuildContext context, _ViewModel viewModel) =>
          _createBody(context, viewModel),
    );
  }

  Widget _createBody(BuildContext context, _ViewModel viewModel) {
    if (widget.widget != null) {
      return Container(
          width: MediaQuery.of(context).size.width,
          height: MediaQuery.of(context).size.height,
          child: Stack(
            children: [
              Padding(
                padding: viewModel.authImpersonateToken != null
                    ? EdgeInsets.only(
                        top: MediaQuery.of(context).padding.top - 24)
                    : EdgeInsets.only(top: 0),
                child: widget.widget!,
              ),
              if (viewModel.authImpersonateToken != null)
                Row(
                  children: [
                    Expanded(
                      child: Material(
                        child: GestureDetector(
                          onTap: () => viewModel.stopImpersonate(),
                          child: AnimatedContainer(
                            color: _color,
                            duration: _myDuration,
                            height: MediaQuery.of(context).padding.top + 30,
                            child: Padding(
                              padding: EdgeInsets.only(
                                  top: MediaQuery.of(context).padding.top - 10),
                              child: Column(
                                mainAxisAlignment: MainAxisAlignment.center,
                                crossAxisAlignment: CrossAxisAlignment.center,
                                children: [
                                  Wrap(
                                    spacing: 4,
                                    alignment: WrapAlignment.center,
                                    crossAxisAlignment:
                                        WrapCrossAlignment.center,
                                    children: [
                                      Icon(Icons.visibility_off,
                                          color: Colors.white, size: 20),
                                      Text("Stop Impersonate",
                                          style: Theme.of(context)
                                              .textTheme
                                              .bodyText1!
                                              .merge(
                                                TextStyle(color: Colors.white),
                                              )),
                                    ],
                                  ),
                                ],
                              ),
                            ),
                          ),
                        ),
                      ),
                    ),
                  ],
                ),
            ],
          ));
    }
    return Text(
      "",
      style: TextStyle(color: Colors.white),
    );
  }
}

class _ViewModel {
  final String? authImpersonateToken;
  final Function stopImpersonate;

  _ViewModel(
    this.authImpersonateToken,
    this.stopImpersonate,
  );

  factory _ViewModel.create(Store<VirtualAccountState> store) {
    return _ViewModel(
      store.state.networkState.authImpersonateToken,
      () => store.dispatch(StopImpersonateAction(
          store.state.networkState.authImpersonateToken ?? "")),
    );
  }
}
