import 'dart:async';

import 'package:account_kit/redux/accountKitState.dart';
import 'package:account_kit/redux/user/userActions.dart';
import 'package:flutter/material.dart';
import 'package:flutter_redux/flutter_redux.dart';
import 'package:redux/redux.dart';

class OpaReady<T extends VirtualAccountState> extends StatefulWidget {
  final String title;
  final String subtitle;
  OpaReady({
    this.title = "We are creating your account...",
    this.subtitle =
        "This step lasts less than a minute.\nYou will be redirected to the app once it is done.",
  });

  @override
  _OpaReadyState createState() => _OpaReadyState<T>();
}

class _OpaReadyState<T extends VirtualAccountState> extends State<OpaReady> {
  Timer? timer;
  int attempt = 0;
  bool debug = false;

  checkOpa(_ViewModel vm) {
    vm.getOpaReady();
    setState(() {
      attempt = attempt + 1;
    });
    if (vm.opaReady) {
      timer?.cancel();
    }
  }

  @override
  void dispose() {
    timer?.cancel();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return StoreConnector<T, _ViewModel>(
      onInitialBuild: (vm) {
        timer = Timer.periodic(new Duration(seconds: 5), (timer) {
          checkOpa(vm);
        });
      },
      onDispose: (vm) => timer?.cancel(),
      converter: (Store<T> store) => _ViewModel.create(store),
      builder: (BuildContext context, _ViewModel viewModel) =>
          _createBody(context, viewModel),
    );
  }

  Widget _createBody(BuildContext context, _ViewModel vm) {
    return GestureDetector(
      onLongPress: () {
        setState(() {
          debug = true;
        });
      },
      child: Center(
          child: Column(
        mainAxisAlignment: MainAxisAlignment.center,
        crossAxisAlignment: CrossAxisAlignment.center,
        children: [
          Padding(
            padding: const EdgeInsets.only(bottom: 16.0),
            child: CircularProgressIndicator(),
          ),
          Text(widget.title, style: Theme.of(context).textTheme.headline5),
          Text(
            widget.subtitle,
            textAlign: TextAlign.center,
          ),
          if (debug)
            Padding(
              padding: const EdgeInsets.only(top: 16.0),
              child: Opacity(
                  opacity: 0.58,
                  child: Text(
                      "Attempt: $attempt. OpaReadyState: ${vm.opaReady ? "True" : "False"}",
                      style: Theme.of(context).textTheme.caption)),
            ),
          OutlinedButton(
            onPressed: () {
              timer?.cancel();
              Navigator.of(context).pop();
            },
            child: Text("Cancel"),
          )
        ],
      )),
    );
  }
}

class _ViewModel {
  final bool opaReady;
  final Function getOpaReady;
  final Function getCurrentUser;

  _ViewModel(
    this.opaReady,
    this.getOpaReady,
    this.getCurrentUser,
  );

  factory _ViewModel.create(Store<VirtualAccountState> store) {
    return _ViewModel(
      store.state.accountKitState.userState.opaReady,
      () => store.dispatch(GetUserOpaReadyAction()),
      () => store.dispatch(GetUserAction()),
    );
  }
}
