import 'package:account_kit/entities/request/request.dart';
import 'package:account_kit/redux/accountKitState.dart';
import 'package:account_kit/redux/user/userActions.dart';
import 'package:email_validator/email_validator.dart';
import 'package:flutter/material.dart';
import 'package:flutter_redux/flutter_redux.dart';
import 'package:puma_network/widget/apiButton.dart';
import 'package:puma_ui/ui/atoms/pumaButton.dart';
import 'package:puma_ui/ui/atoms/smallChip.dart';
import 'package:redux/redux.dart';

class SignupForm<T extends VirtualAccountState> extends StatefulWidget {
  // final void Function(String, String) onSubmit;
  final String? defaultOrganizationToken;
  SignupForm({this.defaultOrganizationToken});

  @override
  _SignupFormStateWidget createState() => _SignupFormStateWidget<T>();
}

/// This is the private State class that goes with SignupForm.
class _SignupFormStateWidget<T extends VirtualAccountState>
    extends State<SignupForm> {
  final _formKey = GlobalKey<FormState>();
  TextEditingController firstNameController = TextEditingController();
  TextEditingController lastNameController = TextEditingController();
  final FocusNode _codeFocus = FocusNode();
  TextEditingController emailController = TextEditingController();
  TextEditingController passwordController = TextEditingController();
  TextEditingController codeController = TextEditingController();
  TextEditingController passwordConfirmationController =
      TextEditingController();
  final FocusNode _firstNameFocus = FocusNode();
  final FocusNode _lastNameFocus = FocusNode();
  final FocusNode _emailFocus = FocusNode();
  final FocusNode _passwordFocus = FocusNode();
  final FocusNode _passwordConfirmationFocus = FocusNode();
  String _password = '';
  String _passwordConfirmation = '';
  bool plEnabled = true;
  String _email = '';
  String _code = '';
  bool preventAutofocus = false;
  // Todo: Get Env default PLEnabled
  bool processing = false;

  handlePasswordChange(value) {
    setState(() {
      _password = value;
    });
  }

  handleEmailChange(value) {
    setState(() {
      _email = value;
    });
  }

  handleCodeChange(value) {
    setState(() {
      _code = value;
    });
  }

  handlePasswordConfirmationChange(value) {
    setState(() {
      _passwordConfirmation = value;
    });
  }

  isDisabled(_ViewModel viewModel) {
    final emailValid = RegExp(
            r"^[a-zA-Z0-9.a-zA-Z0-9.!#$%&'*+-/=?^_`{|}~]+@[a-zA-Z0-9-.-_]+\.[a-zA-Z]+")
        .hasMatch(_email);
    if (plEnabled) {
      return !emailValid;
    } else {
      final bool atLeast8Characters = _password.length >= 8;
      final bool atLeast1Digit = _password.contains(new RegExp(r'\d'));
      final bool atLeast1Letter = _password.contains(new RegExp(r'[A-z]'));
      final bool passwordsMatch = _passwordConfirmation == _password;
      return emailValid &&
          atLeast8Characters &&
          atLeast1Digit &&
          atLeast1Letter &&
          passwordsMatch;
    }
  }

  @override
  Widget build(BuildContext context) {
    final bool atLeast8Characters = _password.length >= 8;
    final bool atLeast1Digit = _password.contains(new RegExp(r'\d'));
    final bool atLeast1Letter = _password.contains(new RegExp(r'[A-z]'));
    final bool passwordsMatch = _passwordConfirmation == _password;
    return (StoreConnector<T, _ViewModel>(
        onDidChange: (vm1, vm2) {
          if (vm2.userRequest != null && !preventAutofocus) {
            preventAutofocus = true;
            _codeFocus.requestFocus();
          }
        },
        converter: (Store<T> store) => _ViewModel.create(store),
        builder: (BuildContext context, _ViewModel viewModel) {
          final _signupPending = viewModel.userRequest != null;
          var buttonLabel = _signupPending ? "Submit Code" : "Create Account";
          return Form(
            key: _formKey,
            child: Wrap(
              runSpacing: 10.0,
              children: <Widget>[
                if (!_signupPending)
                  TextFormField(
                    enabled: !processing,
                    keyboardAppearance: Theme.of(context).brightness,
                    controller: firstNameController,
                    focusNode: _firstNameFocus,
                    autofocus: true,
                    decoration: const InputDecoration(
                      hintText: 'First Name',
                    ),
                    onFieldSubmitted: (value) {
                      _firstNameFocus.unfocus();
                      FocusScope.of(context).requestFocus(_lastNameFocus);
                    },
                    validator: (value) {
                      if (value != null && value.isEmpty) {
                        return 'Please enter your first name';
                      }
                      return null;
                    },
                  ),
                if (!_signupPending)
                  TextFormField(
                    enabled: !processing,
                    keyboardAppearance: Theme.of(context).brightness,
                    controller: lastNameController,
                    focusNode: _lastNameFocus,
                    decoration: const InputDecoration(
                      hintText: 'Last Name',
                    ),
                    onFieldSubmitted: (value) {
                      _lastNameFocus.unfocus();
                      FocusScope.of(context).requestFocus(_emailFocus);
                    },
                    validator: (value) {
                      if (value != null && value.isEmpty) {
                        return 'Please enter your last name';
                      }
                      return null;
                    },
                  ),
                if (!_signupPending)
                  TextFormField(
                    enabled: !processing,
                    keyboardAppearance: Theme.of(context).brightness,
                    controller: emailController,
                    focusNode: _emailFocus,
                    onChanged: this.handleEmailChange,
                    decoration: const InputDecoration(
                      hintText: 'Email',
                    ),
                    onFieldSubmitted: (value) {
                      _emailFocus.unfocus();
                      FocusScope.of(context).requestFocus(_passwordFocus);
                    },
                    validator: (value) {
                      if (value != null && !EmailValidator.validate(value)) {
                        return 'Email is not valid';
                      }
                      return null;
                    },
                  ),
                if (!plEnabled)
                  TextFormField(
                    enabled: !processing,
                    keyboardAppearance: Theme.of(context).brightness,
                    controller: passwordController,
                    onChanged: this.handlePasswordChange,
                    focusNode: _passwordFocus,
                    obscureText: true,
                    decoration: const InputDecoration(
                      hintText: 'Password',
                    ),
                    onFieldSubmitted: (value) {
                      _passwordFocus.unfocus();
                      FocusScope.of(context)
                          .requestFocus(_passwordConfirmationFocus);
                    },
                    validator: (value) {
                      if (value != null && value.isEmpty) {
                        return 'Please enter a password';
                      }
                      return null;
                    },
                  ),
                if (!plEnabled)
                  TextFormField(
                    enabled: !processing,
                    keyboardAppearance: Theme.of(context).brightness,
                    controller: passwordConfirmationController,
                    onChanged: this.handlePasswordConfirmationChange,
                    focusNode: _passwordConfirmationFocus,
                    obscureText: true,
                    decoration: const InputDecoration(
                      hintText: 'Password Confirmation',
                    ),
                    onFieldSubmitted: (value) {
                      _passwordFocus.unfocus();
                    },
                    validator: (value) {
                      if (value != null && value.isEmpty) {
                        return 'Please retype the password';
                      }
                      return null;
                    },
                  ),
                if (_signupPending)
                  Center(
                    child: Card(
                        child: Padding(
                      padding: const EdgeInsets.all(8.0),
                      child: Wrap(
                        crossAxisAlignment: WrapCrossAlignment.center,
                        spacing: 4,
                        children: [
                          Icon(Icons.email_outlined, size: 16),
                          Text(
                              "Check your emails, you should have received a code.",
                              style: Theme.of(context).textTheme.caption),
                        ],
                      ),
                    )),
                  ),
                if (_signupPending)
                  TextFormField(
                    controller: codeController,
                    keyboardAppearance: Theme.of(context).brightness,
                    focusNode: _codeFocus,
                    autofocus: true,
                    onChanged: this.handleCodeChange,
                    keyboardType: TextInputType.number,
                    decoration: const InputDecoration(
                      hintText: 'Enter the 6 digit code',
                    ),
                    validator: (value) {
                      if (value != null && value.isEmpty) {
                        return 'Please enter some text';
                      }
                      return null;
                    },
                  ),
                if (!plEnabled)
                  Center(
                    child: Padding(
                      padding: const EdgeInsets.only(top: 12.0),
                      child: Wrap(
                        runSpacing: 4.0,
                        children: <Widget>[
                          SmallChip(
                            color:
                                atLeast8Characters ? Colors.green : Colors.grey,
                            label: "Min 8 char",
                          ),
                          SmallChip(
                            color: atLeast1Digit ? Colors.green : Colors.grey,
                            label: "Min 1 digit",
                          ),
                          SmallChip(
                            color: atLeast1Letter ? Colors.green : Colors.grey,
                            label: "Min 1 letter",
                          ),
                          SmallChip(
                            color: passwordsMatch ? Colors.green : Colors.grey,
                            label: "Match",
                          )
                        ],
                      ),
                    ),
                  ),
                Padding(
                  padding: const EdgeInsets.symmetric(vertical: 16.0),
                  child: Row(
                    mainAxisAlignment: MainAxisAlignment.center,
                    crossAxisAlignment: CrossAxisAlignment.center,
                    children: [
                      if (viewModel.userRequest != null)
                        PumaButton(
                          title: "Cancel",
                          onPressed: () {
                            viewModel.clearUserRequest();
                            _codeFocus.unfocus();
                            _emailFocus.requestFocus();
                          },
                        ),
                      ApiButton(
                          label: buttonLabel,
                          disabled: isDisabled(viewModel),
                          finishCallback: () {
                            processing = false;
                            _codeFocus.requestFocus();
                          },
                          color: Theme.of(context).accentColor,
                          apiCall: () {
                            FocusScope.of(context).unfocus();
                            if (plEnabled) {
                              if (_formKey.currentState!.validate()) {
                                if (viewModel.userRequest != null) {
                                  return viewModel
                                      .postPLSignin(this.codeController.text);
                                } else {
                                  return viewModel.plSignup(
                                    this.firstNameController.text,
                                    this.lastNameController.text,
                                    this.emailController.text,
                                    widget.defaultOrganizationToken,
                                  );
                                }
                              }
                            } else {
                              if (_formKey.currentState!.validate()) {
                                return viewModel.signup(
                                  this.firstNameController.text,
                                  this.lastNameController.text,
                                  this.emailController.text,
                                  this.passwordController.text,
                                  this.passwordConfirmationController.text,
                                  widget.defaultOrganizationToken,
                                );
                              }
                            }
                            return null;
                          }),
                    ],
                  ),
                ),
              ],
            ),
          );
        }));
  }
}

class _ViewModel {
  final Function signup;
  final Function plSignup;
  final Request? userRequest;
  final Function(String) postPLSignin;
  final Function() clearUserRequest;

  _ViewModel(this.signup, this.plSignup, this.userRequest, this.postPLSignin,
      this.clearUserRequest);

  factory _ViewModel.create(Store<VirtualAccountState> store) {
    return _ViewModel(
        (firstName, lastName, email, password, passwordConfirmation,
                organizationToken) =>
            PostSignup(
                email: email,
                firstName: firstName,
                lastName: lastName,
                password: password,
                passwordConfirmation: passwordConfirmation,
                organizationToken: organizationToken),
        (firstName, lastName, email, organizationToken) => PostPLSignup(
              email: email,
              firstName: firstName,
              lastName: lastName,
              organizationToken: organizationToken,
            ),
        store.state.accountKitState.userState.request,
        (code) => PostPLSignin(code),
        () => store.dispatch(ClearUserRequestAction()));
  }
}
